dnl @synopsis AC_PATH_GENERIC(LIBRARY [, MINIMUM-VERSION [, ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl
dnl Runs a LIBRARY-config script and defines LIBRARY_CFLAGS and LIBRARY_LIBS
dnl
dnl The script must support `--cflags' and `--libs' args.
dnl If MINIMUM-VERSION is specified, the script must also support the
dnl `--version' arg.
dnl If the `--with-library-[exec-]prefix' arguments to ./configure are given,
dnl it must also support `--prefix' and `--exec-prefix'.
dnl (In other words, it must be like gtk-config.)
dnl
dnl For example:
dnl
dnl    AC_PATH_GENERIC(Foo, 1.0.0)
dnl
dnl would run `foo-config --version' and check that it is at least 1.0.0
dnl
dnl If so, the following would then be defined:
dnl
dnl    FOO_CFLAGS to `foo-config --cflags`
dnl    FOO_LIBS   to `foo-config --libs`
dnl
dnl At present there is no support for additional "MODULES" (see AM_PATH_GTK)
dnl (shamelessly stolen from gtk.m4 and then hacked around a fair amount)
dnl
dnl @author Angus Lees <gusl@cse.unsw.edu.au>
dnl @version $Id: ferrismacros.m4,v 1.5 2004/02/15 12:08:16 monkeyiq Exp $

AC_DEFUN(AC_PATH_GENERIC,
[dnl
dnl we're going to need uppercase, lowercase and user-friendly versions of the
dnl string `LIBRARY'
pushdef([UP], translit([$1], [a-z], [A-Z]))dnl
pushdef([DOWN], translit([$1], [A-Z], [a-z]))dnl

dnl
dnl Get the cflags and libraries from the LIBRARY-config script
dnl
AC_ARG_WITH(DOWN-prefix,[  --with-]DOWN[-prefix=PFX       Prefix where $1 is installed (optional)],
        DOWN[]_config_prefix="$withval", DOWN[]_config_prefix="")
AC_ARG_WITH(DOWN-exec-prefix,[  --with-]DOWN[-exec-prefix=PFX Exec prefix where $1 is installed (optional)],
        DOWN[]_config_exec_prefix="$withval", DOWN[]_config_exec_prefix="")

  if test x$DOWN[]_config_exec_prefix != x ; then
     DOWN[]_config_args="$DOWN[]_config_args --exec-prefix=$DOWN[]_config_exec_prefix"
     if test x${UP[]_CONFIG+set} != xset ; then
       UP[]_CONFIG=$DOWN[]_config_exec_prefix/bin/DOWN-config
     fi
  fi
  if test x$DOWN[]_config_prefix != x ; then
     DOWN[]_config_args="$DOWN[]_config_args --prefix=$DOWN[]_config_prefix"
     if test x${UP[]_CONFIG+set} != xset ; then
       UP[]_CONFIG=$DOWN[]_config_prefix/bin/DOWN-config
     fi
  fi

  AC_PATH_PROG(UP[]_CONFIG, DOWN-config, no)
  ifelse([$2], ,
     AC_MSG_CHECKING(for $1),
     AC_MSG_CHECKING(for $1 - version >= $2)
  )
  no_[]DOWN=""
  if test "$UP[]_CONFIG" = "no" ; then
     no_[]DOWN=yes
  else
     UP[]_CFLAGS="`$UP[]_CONFIG $DOWN[]_config_args --cflags`"
     UP[]_LIBS="`$UP[]_CONFIG $DOWN[]_config_args --libs`"
     ifelse([$2], , ,[
        DOWN[]_config_major_version=`$UP[]_CONFIG $DOWN[]_config_args \
         --version | sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
        DOWN[]_config_minor_version=`$UP[]_CONFIG $DOWN[]_config_args \
         --version | sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
        DOWN[]_config_micro_version=`$UP[]_CONFIG $DOWN[]_config_args \
         --version | sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
        DOWN[]_wanted_major_version="regexp($2, [\<\([0-9]*\)], [\1])"
        DOWN[]_wanted_minor_version="regexp($2, [\<\([0-9]*\)\.\([0-9]*\)], [\2])"
        DOWN[]_wanted_micro_version="regexp($2, [\<\([0-9]*\).\([0-9]*\).\([0-9]*\)], [\3])"

        # Compare wanted version to what config script returned.
        # If I knew what library was being run, i'd probably also compile
        # a test program at this point (which also extracted and tested
        # the version in some library-specific way)
        if test "$DOWN[]_config_major_version" -lt \
                        "$DOWN[]_wanted_major_version" \
          -o \( "$DOWN[]_config_major_version" -eq \
                        "$DOWN[]_wanted_major_version" \
            -a "$DOWN[]_config_minor_version" -lt \
                        "$DOWN[]_wanted_minor_version" \) \
          -o \( "$DOWN[]_config_major_version" -eq \
                        "$DOWN[]_wanted_major_version" \
            -a "$DOWN[]_config_minor_version" -eq \
                        "$DOWN[]_wanted_minor_version" \
            -a "$DOWN[]_config_micro_version" -lt \
                        "$DOWN[]_wanted_micro_version" \) ; then
          # older version found
          no_[]DOWN=yes
          echo -n "*** An old version of $1 "
          echo -n "($DOWN[]_config_major_version"
          echo -n ".$DOWN[]_config_minor_version"
          echo    ".$DOWN[]_config_micro_version) was found."
          echo -n "*** You need a version of $1 newer than "
          echo -n "$DOWN[]_wanted_major_version"
          echo -n ".$DOWN[]_wanted_minor_version"
          echo    ".$DOWN[]_wanted_micro_version."
          echo "***"
          echo "*** If you have already installed a sufficiently new version, this error"
          echo "*** probably means that the wrong copy of the DOWN-config shell script is"
          echo "*** being found. The easiest way to fix this is to remove the old version"
          echo "*** of $1, but you can also set the UP[]_CONFIG environment to point to the"
          echo "*** correct copy of DOWN-config. (In this case, you will have to"
          echo "*** modify your LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf"
          echo "*** so that the correct libraries are found at run-time)"
        fi
     ])
  fi
  if test "x$no_[]DOWN" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$3], , :, [$3])
  else
     AC_MSG_RESULT(no)
     if test "$UP[]_CONFIG" = "no" ; then
       echo "*** The DOWN-config script installed by $1 could not be found"
       echo "*** If $1 was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the UP[]_CONFIG environment variable to the"
       echo "*** full path to DOWN-config."
     fi
     UP[]_CFLAGS=""
     UP[]_LIBS=""
     ifelse([$4], , :, [$4])
  fi
  AC_SUBST(UP[]_CFLAGS)
  AC_SUBST(UP[]_LIBS)

  popdef([UP])
  popdef([DOWN])
])

################################################################################
################################################################################
################################################################################

#
# Common macros used by many configure.in scripts in the ferris suite.
#

###############################################################################
###############################################################################
###############################################################################
# Test for stlport 4.5
###############################################################################

dnl
dnl
dnl See AM_FERRIS_STLPORT() for the macro you want to call externally.
dnl
dnl

STLPORT_IO64_CFLAGS=" -D_GNU_SOURCE -D_FILE_OFFSET_BITS=64 "

dnl
dnl
dnl AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]])
dnl Used internally to try to link an STLPort using C++ application
dnl using the STLPORT_CFLAGS and STLPORT_LIBS. 
dnl STLPORT_IOSIZE will be set to either 32 or 64 depending on what 
dnl width of IO the found STLPort supports and STLPORT_CFLAGS may be adjusted
dnl for 64 bit building.
dnl 
dnl the STLPORT_CFLAGS etc are all cleared after a failed test.
dnl
AC_DEFUN(AM_FERRIS_STLPORT_INTERNAL_TRYLINK,
[dnl
dnl
	AC_LANG_CPLUSPLUS
	CXXFLAGS_cache=$CXXFLAGS
	CXXFLAGS=" $CXXFLAGS $STLPORT_CFLAGS "
	LDFLAGS_cache=$LDFLAGS
	LDFLAGS=" $LDFLAGS $STLPORT_LIBS "

	AC_TRY_LINK([
		#include <hash_map>
		],
		[
		std::hash_map<int,int> hm;
		hm[5] = 6;
		],
	       	[ferris_stlport_internal_trylink=yes; STLPORT_IOSIZE=32 ],
	       	[ferris_stlport_internal_trylink=no] )

	LDFLAGS=$LDFLAGS_cache
	CXXFLAGS=$CXXFLAGS_cache
	AC_LANG_C

	if test x"$ferris_stlport_internal_trylink" = xno; then
		AC_LANG_CPLUSPLUS
		CXXFLAGS_cache=$CXXFLAGS
		CXXFLAGS=" $CXXFLAGS $STLPORT_IO64_CFLAGS $STLPORT_CFLAGS "
		LDFLAGS_cache=$LDFLAGS
		LDFLAGS=" $LDFLAGS $STLPORT_LIBS "

		AC_TRY_LINK([
			#include <hash_map>
			],
			[
			std::hash_map<int,int> hm;
			hm[5] = 6;
			],
		       	[ferris_stlport_internal_trylink=yes; STLPORT_IOSIZE=64 ],
	       		[ferris_stlport_internal_trylink=no] )

		LDFLAGS=$LDFLAGS_cache
		CXXFLAGS=$CXXFLAGS_cache
		AC_LANG_C
	fi

	if test x"$ferris_stlport_internal_trylink" = xyes; then
	     ifelse([$1], , :, [$1])     
	else
	     ifelse([$2], , :, [$2])     
		if test x"$have_stlport" = xno; then
			STLPORT_CFLAGS=""
			STLPORT_LDFLAGS=""
	 		STLPORT_LIB=""
			STLPORT_LIBS=""
		fi
	fi
])

dnl AM_FERRIS_STLPORT([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate STLPort for installation. 
dnl ie. default is to REQUIRE STLPort MINIMUM-VERSION or stop running.
dnl
dnl MINIMUM-VERSION must be a three part value, like 4.5.0
dnl
dnl Test for STLPort, and define STLPORT_CFLAGS, STLPORT_LIBS and STLPORT_IOSIZE
dnl other side effects include
dnl AM_CONDITIONAL( HAVE_STLPORT, 1 or 0 )
dnl AC_SUBST( STLPORT_CFLAGS )
dnl AC_SUBST( STLPORT_LIBS )
dnl 
dnl if( success ) 
dnl    AC_DEFINE( HAVE_STLPORT )
dnl    AC_DEFINE( STLPORT_IOSIZE )
dnl
AC_DEFUN(AM_FERRIS_STLPORT,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl

stlport_required_version=$1
have_stlport=no

AC_ARG_WITH(stlport,
        [  --with-stlport=DIR          use stlport 4.5+ install rooted at <DIR>],
        [STLPORT_CFLAGS=" -I$withval/stlport "
	 STLPORT_LDFLAGS=" -L$withval/lib "
 	 STLPORT_LIB=" -lstlport_gcc_debug -lpthread "
	 STLPORT_LIBS=" -L$withval/lib ${STLPORT_LIB} "
	 stlport_try_trivial_compile=yes
        ])
if test x"$stlport_try_trivial_compile" = xyes; then
	AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [have_stlport=yes], [have_stlport=no]  )
fi

if test x"$have_stlport" = xno; then

	package=stlport
	version=4.5.3
	PKG_CHECK_MODULES(STLPORT, $package >= $version, [ have_stlport=yes ], [foo=1] )
fi

if test x"$have_stlport" = xno; then

	AC_LANG_CPLUSPLUS
	STLPORT_IO64_CFLAGS=" -D_GNU_SOURCE -D_FILE_OFFSET_BITS=64 "

	AC_CHECK_PROG( have_stlportcfg, stlport-config, yes, no )
	
	if test "$have_stlportcfg" = yes; then

#		AC_PATH_GENERIC( STLPORT, 4.5, [ have_stlport=yes ], [foo=2] )
		stlport_installed_version=`stlport-config --version`

		# Calculate the available version number
		[f_tmp=( `echo $stlport_installed_version | sed 's/[^0-9]\+/ /g'` )]
		[f_tmp=$(( 1000000 * ${f_tmp[0]:-0} + 1000 * ${f_tmp[1]:-0} + ${f_tmp[2]:-0} ))]

		[freq_version=( `echo $stlport_required_version | sed 's/[^0-9]\+/ /g'` )]
		[freq_version=$(( 1000000 * ${freq_version[0]:-0} + 1000 * ${freq_version[1]:-0} + ${freq_version[2]:-0} ))]

		if test $freq_version -gt $f_tmp ; then
			AC_MSG_WARN([STLPort version $1 is required, you have $stlport_installed_version])
		else
			have_stlport=yes
			STLPORT_LIBS=" `stlport-config --libs` -lpthread "
			STLPORT_CFLAGS=" `stlport-config --cflags` "
			AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [have_stlport=yes], [have_stlport=no]  )
		fi
	fi
fi

if test x"$have_stlport" = xno; then
	STLPORT_CFLAGS=" -I/usr/local/STLport-4.5/stlport "
	STLPORT_LIB=" -lstlport_gcc_debug -lpthread "
	STLPORT_LIBS=" -L/usr/local/STLport-4.5/lib ${STLPORT_LIB} "

	AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [have_stlport=yes], [have_stlport=no]  )
fi

if test x"$have_stlport" = xno; then
	STLPORT_CFLAGS=" -I/usr/local/include/stlport "
	STLPORT_LIB=" -lstlport_gcc_debug -lpthread "
	STLPORT_LIBS=" -L/usr/local/lib ${STLPORT_LIB} "

	AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [have_stlport=yes], [have_stlport=no]  )
fi

if test x"$have_stlport" = xno; then
	STLPORT_CFLAGS=" -I/usr/include/stlport "
	STLPORT_LIB=" -lstlport_gcc_debug -lpthread "
	STLPORT_LIBS=" -L/usr/lib ${STLPORT_LIB} "

	AM_FERRIS_STLPORT_INTERNAL_TRYLINK( [have_stlport=yes], [have_stlport=no]  )
fi

dnl
dnl just make sure of the assertion that we have a valid STLPort
dnl
if test x"$have_stlport" = xyes; then

	AM_FERRIS_STLPORT_INTERNAL_TRYLINK( 
	[
		AC_DEFINE( HAVE_STLPORT )
		AC_DEFINE( STLPORT_IOSIZE )

		echo "Found an STLport that meets required needs..."
		echo "  STLPORT_CFLAGS: $STLPORT_CFLAGS "
		echo "  STLPORT_LIBS:   $STLPORT_LIBS "

		# success
		ifelse([$2], , :, [$2])
	], 
	[
		# fail
		ifelse([$3], , 
		[
			echo ""
			echo "STLPort $version can not be detected on your system. "
			echo ""
			echo "Please make sure that STLPort with IOStreams"
			echo "support is available on your machine before "
			echo "trying again. "
			echo ""
			echo "get it from the URLs below"
			echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
			echo "  http://www.stlport.com/download.html"
			AC_MSG_ERROR([Fatal Error: no STLPort $version or later found.])	
		], 
	[$3])     
	] )
else
	ifelse([$3], , 
	[
		echo ""
		echo "STLPort $version can not be detected on your system. "
		echo ""
		echo "Please make sure that STLPort with IOStreams"
		echo "support is available on your machine before "
		echo "trying again. "
		echo ""
		echo "get it from the URLs below"
		echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
		echo "  http://www.stlport.com/download.html"
		AC_MSG_ERROR([Fatal Error: no STLPort $version or later found.])	
	], 
	[$3])     
fi

AC_SUBST( STLPORT_CFLAGS )
AC_SUBST( STLPORT_LDFLAGS )
AC_SUBST( STLPORT_LIBS )
AM_CONDITIONAL(HAVE_STLPORT, test x"$have_stlport" = xyes)

AC_LANG_C
])





dnl AM_FERRIS_FERRIS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate libferris for installation. 
dnl ie. default is to REQUIRE libferris MINIMUM-VERSION or stop running.
dnl
dnl FERRIS_CFLAGS and FERRIS_LIBS are set and AC_SUBST()ed when library is found.
dnl LIBFERRIS_CFLAGS and LIBFERRIS_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_FERRIS,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
have_package=no
required_version=$1

package=ferris
version=$required_version
PKG_CHECK_MODULES(FERRIS, $package >= $version,
[
	AC_DEFINE( HAVE_FERRIS )
	have_package=yes

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
  		echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "this should be on the freshrpms.net website"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AM_CONDITIONAL(HAVE_FERRIS, test x"$have_package" = xyes)
AC_SUBST(FERRIS_CFLAGS)
AC_SUBST(FERRIS_LIBS)
LIBFERRIS_CFLAGS="$FERRIS_CFLAGS"
LIBFERRIS_LIBS="$FERRIS_LIBS"
AC_SUBST(LIBFERRIS_CFLAGS)
AC_SUBST(LIBFERRIS_LIBS)
])


dnl AM_FERRIS_FERRISUI([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate libferris for installation. 
dnl ie. default is to REQUIRE libferris MINIMUM-VERSION or stop running.
dnl
dnl FERRISUI_CFLAGS and FERRISUI_LIBS are set and AC_SUBST()ed when library is found.
dnl LIBFERRISUI_CFLAGS and LIBFERRISUI_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_FERRISUI,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
have_package=no
required_version=$1

package=ferrisui
version=$required_version
PKG_CHECK_MODULES(FERRISUI, $package >= $version,
[
	AC_DEFINE( HAVE_FERRISUI )
	have_package=yes

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
  		echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "this should be on the freshrpms.net website"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AM_CONDITIONAL(HAVE_FERRISUI, test x"$have_package" = xyes)
AC_SUBST(FERRISUI_CFLAGS)
AC_SUBST(FERRISUI_LIBS)
LIBFERRISUI_CFLAGS="$FERRISUI_CFLAGS"
LIBFERRISUI_LIBS="$FERRISUI_LIBS"
AC_SUBST(LIBFERRISUI_CFLAGS)
AC_SUBST(LIBFERRISUI_LIBS)
])


dnl AM_FERRIS_SIGC([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate sigc++ for installation. 
dnl ie. default is to REQUIRE sigc++ MINIMUM-VERSION or stop running.
dnl
dnl SIGC_CFLAGS and SIGC_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_SIGC,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
have_package=no
sigc_required_version=$1

package=sigc++-1.2
version=$sigc_required_version
PKG_CHECK_MODULES(SIGC, $package >= $version,
[
	AC_DEFINE( HAVE_SIGC )

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
  		echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "this should be on the freshrpms.net website"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])

AC_SUBST(SIGC_CFLAGS)
AC_SUBST(SIGC_LIBS)
])


dnl AM_FERRIS_LOKI([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate ferrisloki for installation. 
dnl ie. default is to REQUIRE ferrisloki MINIMUM-VERSION or stop running.
dnl
dnl LOKI_CFLAGS and LOKI_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_LOKI,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
required_version=$1

package=ferrisloki
version=$required_version
PKG_CHECK_MODULES(LOKI, $package >= $version, 
[
	AC_DEFINE( HAVE_LOKI )

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
	  	echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "get it from the URL"
		echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AC_SUBST(LOKI_CFLAGS)
AC_SUBST(LOKI_LIBS)
])


dnl AM_FERRIS_STREAMS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate ferrisstreams for installation. 
dnl ie. default is to REQUIRE ferrisstreams MINIMUM-VERSION or stop running.
dnl
dnl STREAMS_CFLAGS and STREAMS_LIBS are set and AC_SUBST()ed when library is found.
dnl FSTREAM_CFLAGS and FSTREAM_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_STREAMS,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
required_version=$1

package=ferrisstreams
version=$required_version
PKG_CHECK_MODULES(STREAMS, $package >= $version, 
[
	AC_DEFINE( HAVE_STREAMS )

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
	  	echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "get it from the URL"
		echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AC_SUBST(STREAMS_CFLAGS)
AC_SUBST(STREAMS_LIBS)

FSTREAM_CFLAGS=$STREAMS_CFLAGS
FSTREAM_LIBS=$STREAMS_LIBS
AC_SUBST(FSTREAM_CFLAGS)
AC_SUBST(FSTREAM_LIBS)
])


dnl AM_FERRIS_STLDB4([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate stldb4 for installation. 
dnl ie. default is to REQUIRE stldb4 MINIMUM-VERSION or stop running.
dnl
dnl STLDB4_CFLAGS and STLDB4_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_STLDB4,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
required_version=$1

package=stldb4
version=$required_version
PKG_CHECK_MODULES(STLDB4, $package >= $version, 
[
	AC_DEFINE( HAVE_STLDB4 )

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
	  	echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "get it from the URL"
		echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AC_SUBST(STLDB4_CFLAGS)
AC_SUBST(STLDB4_LIBS)
])


dnl AM_FERRIS_FAMPP2([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate fampp2 for installation. 
dnl ie. default is to REQUIRE fampp2 MINIMUM-VERSION or stop running.
dnl
dnl FAMPP2_CFLAGS and FAMPP2_LIBS are set and AC_SUBST()ed when library is found.
dnl
AC_DEFUN(AM_FERRIS_FAMPP2,
[dnl 
dnl Get the cflags and libraries from pkg-config or x-config
dnl
required_version=$1

package=fampp2
version=$required_version
AC_PATH_GENERIC(FAMPP2, $version, 
[
	AC_DEFINE( HAVE_FAMPP2 )

	# success
	ifelse([$2], , :, [$2])
],
[
	ifelse([$3], , 
	[
	  	echo ""
		echo "latest version of $package required. ($version or better) "
		echo ""
		echo "get it from the URL"
		echo "http://sourceforge.net/project/showfiles.php?group_id=16036"
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
	])
AC_SUBST(FAMPP2_CFLAGS)
AC_SUBST(FAMPP2_LIBS)
])


dnl
dnl
dnl AM_FERRIS_INTERNAL_TRYLINK( CFLAGS, LIBS, HEADERS, BODY, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]])
dnl Used internally to try to link to a library using C++ application
dnl using the CFLAGS and LIBS. 
dnl
AC_DEFUN(AM_FERRIS_INTERNAL_TRYLINK,
[dnl
dnl
	AC_LANG_CPLUSPLUS
	CXXFLAGS_cache=$CXXFLAGS
	CXXFLAGS=" $CXXFLAGS $1 "
	LDFLAGS_cache=$LDFLAGS
	LDFLAGS=" $LDFLAGS $2 "

	AC_TRY_LINK([
		$3
		],
		[
		$4
		],
	       	[trylink_passed=yes ],
	       	[trylink_passed=no] )

	LDFLAGS=$LDFLAGS_cache
	CXXFLAGS=$CXXFLAGS_cache
	AC_LANG_C

	if test x"$trylink_passed" = xyes; then
	     ifelse([$5], , :, [$5])     
	else
	     ifelse([$6], , :, [$6])     
	fi
])

dnl
dnl
dnl AM_FERRIS_INTERNAL_TRYRUN( CFLAGS, LIBS, HEADERS, BODY, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]])
dnl Used internally to try to link to a library using C++ application
dnl using the CFLAGS and LIBS. 
dnl
AC_DEFUN(AM_FERRIS_INTERNAL_TRYRUN,
[dnl
dnl
	AC_LANG_CPLUSPLUS
	CXXFLAGS_cache=$CXXFLAGS
	CXXFLAGS=" $CXXFLAGS $1 "
	LDFLAGS_cache=$LDFLAGS
	LDFLAGS=" $LDFLAGS $2 "

	AC_TRY_RUN([
		$3
		
		int main( int argc, char** argv ) {
			$4
			return 0; }
		],
	       	[trylink_passed=yes ],
	       	[trylink_passed=no] )

	LDFLAGS=$LDFLAGS_cache
	CXXFLAGS=$CXXFLAGS_cache
	AC_LANG_C

	if test x"$trylink_passed" = xyes; then
	     ifelse([$5], , :, [$5])     
	else
	     ifelse([$6], , :, [$6])     
	fi
])


dnl AM_FERRIS_XERCESC([EXACT-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate xerces-c for installation. 
dnl ie. default is to REQUIRE xerces-c EXACT-VERSION or stop running.
dnl
dnl XERCESC_CFLAGS and XERCESC_LIBS are set and AC_SUBST()ed when library is found.
dnl XML4C_CFLAGS   and XML4C_LIBS   are set and AC_SUBST()ed when library is found.
dnl AC_DEFINE(HAVE_XML4C) and AC_DEFINE(HAVE_XERCESC)
dnl
AC_DEFUN(AM_FERRIS_XERCESC,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
have_package=no
required_version=$1
have_xml4c=no

package=xerces-c
version=$required_version
PKG_CHECK_MODULES(XERCESC, $package = $version, [ have_package=yes ], [ have_package=no ] )
dnl if test x"$have_package" = xno; then
dnl 	AC_PATH_GENERIC(XERCES-C, $version, [ have_package=yes ], [ have_package=no ] )
dnl fi

INCLUDES="$(cat <<-HEREDOC
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XercesVersion.hpp>
#include <iostream.h> 
XERCES_CPP_NAMESPACE_USE
HEREDOC
)"
PROGRAM="$(cat <<-HEREDOC
    // Initialize the XML4C2 system.
    try
    {
        XMLPlatformUtils::Initialize();
    }

    catch(const XMLException& toCatch)
    {
        char *pMsg = XMLString::transcode(toCatch.getMessage());
        cerr << "Error during Xerces-c Initialization.\n"
             << "  Exception message:"
             << pMsg;
        XMLString::release(&pMsg);
        return 1;
    }

    
    if( XERCES_VERSION_MAJOR != 2 && XERCES_VERSION_MINOR != 2 )
    {
        return 1;
    }
HEREDOC
)"

if test x"$have_package" = xno; then
AC_ARG_WITH(xercesc,
        [  --with-xercesc=DIR          use xercesc $version install rooted at <DIR>],
        [XERCESC_CFLAGS=" -I$withval/xercesc "
	 XERCESC_LIBS=" -L$withval/lib -lxerces-c " 
	 AM_FERRIS_INTERNAL_TRYRUN( [$XERCESC_CFLAGS], [$XERCESC_LIBS], 
					[ $INCLUDES ], [$PROGRAM],
					[have_package=yes], [have_package=no] )
	])
fi

# try to hit it directly.
if test x"$have_package" = xno; then
	XERCESC_CFLAGS=" -I/usr/include/xercesc "
	XERCESC_LIBS=" -L/usr/lib -lxerces-c "
	AM_FERRIS_INTERNAL_TRYRUN( [$XERCESC_CFLAGS], [$XERCESC_LIBS], 
				[ $INCLUDES ], [$PROGRAM],
				[have_package=yes], [have_package=no] )
fi
# try to hit it directly.
if test x"$have_package" = xno; then
	XERCESC_CFLAGS=" -I/usr/local/include/xercesc "
	XERCESC_LIBS=" -L/usr/local/lib -lxerces-c "
	AM_FERRIS_INTERNAL_TRYRUN( [$XERCESC_CFLAGS], [$XERCESC_LIBS], 
				[ $INCLUDES ], [$PROGRAM],
				[have_package=yes], [have_package=no] )
fi


if test x"$have_package" = xyes; then
	have_xml4c=yes
	AC_DEFINE( HAVE_XERCESC )
	AC_DEFINE( HAVE_XML4C )

	echo "Found an xerces-c that meets required needs..."
	echo "  XERCESC_CFLAGS: $XERCESC_CFLAGS "
	echo "  XERCESC_LIBS:   $XERCESC_LIBS "

	# success
	ifelse([$2], , :, [$2])
else
	ifelse([$3], , 
	[
		have_xml4c=no
		echo ""
		echo "explicit version ($version) of $package required. "
		echo ""
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
fi

AC_SUBST(XERCESC_CFLAGS)
AC_SUBST(XERCESC_LIBS)

AM_CONDITIONAL(HAVE_XML4C, test x"$have_xml4c" = xyes)
XML4C_CFLAGS=$XERCESC_CFLAGS
XML4C_LIBS=$XERCESC_LIBS
AC_SUBST(XML4C_CFLAGS)
AC_SUBST(XML4C_LIBS)

])


dnl AM_FERRIS_XALAN([EXACT-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to AC_MSG_ERROR() with a description of where
dnl to locate xerces-c for installation. 
dnl ie. default is to REQUIRE xerces-c EXACT-VERSION or stop running.
dnl
dnl XALAN_CFLAGS and XALAN_LIBS are set and AC_SUBST()ed when library is found.
dnl XML4C_CFLAGS   and XML4C_LIBS   are set and AC_SUBST()ed when library is found.
dnl AC_DEFINE(HAVE_XML4C) and AC_DEFINE(HAVE_XALAN)
dnl
AC_DEFUN(AM_FERRIS_XALAN,
[dnl 
dnl Get the cflags and libraries from pkg-config, stlport-config or attempt to
dnl detect the STLPort on the users system.
dnl
required_version=$1
have_xalan=no

package=xalan-c
version=$required_version

PKG_CHECK_MODULES(XALAN, $package >= $version, [ have_xalan=yes ],  [ have_xalan=no ] )

INCLUDES="$(cat <<-HEREDOC
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XercesVersion.hpp>
#include <Include/XalanVersion.hpp>
#include <iostream.h> 
XERCES_CPP_NAMESPACE_USE
HEREDOC
)"
PROGRAM="$(cat <<-HEREDOC
    // Initialize the XML4C2 system.
    try
    {
        XMLPlatformUtils::Initialize();
    }

    catch(const XMLException& toCatch)
    {
        char *pMsg = XMLString::transcode(toCatch.getMessage());
        cerr << "Error during Xerces-c Initialization.\n"
             << "  Exception message:"
             << pMsg;
        XMLString::release(&pMsg);
        return 1;
    }

    
    if( XALAN_VERSION_MAJOR != 1 && XALAN_VERSION_MINOR != 5 )
    {
        return 1;
    }
HEREDOC
)"


if test x"$have_xalan" = xno; then
AC_ARG_WITH(xalan,
        [  --with-xalan=DIR          use xalan $version install rooted at <DIR>],
        [XALAN_CFLAGS=" -I$withval/xalan-c1.5 "
	 XALAN_LIBS=" -L$withval/lib -lxalan-c1_5_0 " 
	 AM_FERRIS_INTERNAL_TRYRUN( [$XALAN_CFLAGS], [$XALAN_LIBS], 
					[ $INCLUDES ], [$PROGRAM],
					[have_xalan=yes], [have_xalan=no] )
	])
fi

# try to hit it directly.
if test x"$have_xalan" = xno; then
	XALAN_CFLAGS=" -I/usr/include/xalan-c1.5 "
	XALAN_LIBS=" -L/usr/lib -lxalan-c1_5_0 "
	AM_FERRIS_INTERNAL_TRYRUN( [$XALAN_CFLAGS], [$XALAN_LIBS], 
				[ $INCLUDES ], [$PROGRAM],
				[have_xalan=yes], [have_xalan=no] )
fi
# try to hit it directly.
if test x"$have_xalan" = xno; then
	XALAN_CFLAGS=" -I/usr/include/xalan-c "
	XALAN_LIBS=" -L/usr/lib -lxalan-c "
	AM_FERRIS_INTERNAL_TRYRUN( [$XALAN_CFLAGS], [$XALAN_LIBS], 
				[ $INCLUDES ], [$PROGRAM],
				[have_xalan=yes], [have_xalan=no] )
fi
# try to hit it directly.
if test x"$have_xalan" = xno; then
	XALAN_CFLAGS=" -I/usr/local/include/xalan-c1.5 "
	XALAN_LIBS=" -L/usr/local/lib -lxalan-c1_5_0 "
	AM_FERRIS_INTERNAL_TRYRUN( [$XALAN_CFLAGS], [$XALAN_LIBS], 
				[ $INCLUDES ], [$PROGRAM],
				[have_xalan=yes], [have_xalan=no] )
fi


if test x"$have_xalan" = xyes; then

	have_xalan=yes
	AC_DEFINE( HAVE_XALAN )

	echo "Found an xalan-c that meets required needs..."
	echo "  XALAN_CFLAGS: $XALAN_CFLAGS "
	echo "  XALAN_LIBS:   $XALAN_LIBS "

	# success
	ifelse([$2], , :, [$2])

else
	ifelse([$3], , 
	[
		echo ""
		echo "version ($version) or later of $package required. "
		echo ""
		AC_MSG_ERROR([Fatal Error: no correct $package found.])	
	], 
	[$3])     
fi


AM_CONDITIONAL(HAVE_XALAN, test x"$have_xalan" = xyes)
AC_SUBST(XALAN_CFLAGS)
AC_SUBST(XALAN_LIBS)

])


dnl ######################################################################
dnl ######################################################################
dnl ######################################################################
dnl ######################################################################


dnl AM_FERRIS_DTL([EXACT-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl
dnl The default ACTION-IF-NOT-FOUND is to put a message for the user to see that
dnl this module was not found and thus code written for it is not being compiled in.
dnl
dnl DTL_CFLAGS and DTL_LIBS are set and AC_SUBST()ed when library is found.
dnl AC_DEFINE(HAVE_DTL) and AM_CONDITIONAL(HAVE_DTL)
dnl
AC_DEFUN(AM_FERRIS_DTL,
[dnl 
dnl
required_version=$1
have_dtl=no

package=dtl
version=$required_version

PKG_CHECK_MODULES(DTL, $package >= $version, [ have_dtl=yes ],  [ have_dtl=no ] )

if test x"$have_dtl" = xyes; then

	have_dtl=yes
	AC_DEFINE( HAVE_DTL )

	echo "Found a DTL ODBC library that meets required needs..."
	echo "  DTL_CFLAGS: $DTL_CFLAGS "
	echo "  DTL_LIBS:   $DTL_LIBS "

	# success
	ifelse([$2], , :, [$2])

else
	ifelse([$3], , 
	[
	echo "Support for DTL version ($version) not being built... "
	], 
	[$3])     
fi


AM_CONDITIONAL(HAVE_DTL, test x"$have_dtl" = xyes)
AC_SUBST(DTL_CFLAGS)
AC_SUBST(DTL_LIBS)
])
